@@ -12,9 +12,6 @@ std::ostringstream result_stream;
 
 struct foo
 {
-  // choose a type that can hold all return values
-  using result_type = int;
-
   int operator()(int i)
   {
     result_stream << "foo(int " << i << ") ";
@@ -36,8 +33,6 @@ struct foo
 
 struct foo_void
 {
-  using result_type = void;
-
   void operator()(int i) { result_stream << "foo_void(int " << i << ")"; }
 };
 
@@ -121,7 +116,7 @@ main(int argc, char* argv[])
   util->check_result(result_stream, "foo_void(int 12)");
 
   // function pointer instead of functor
-  sigc::bind (&bar, 13, 14)();
+  sigc::bind(&bar, 13, 14)();
   util->check_result(result_stream, "bar(int 13, int 14) ");
 
   // method pointer instead of functor
@@ -138,7 +133,7 @@ main(int argc, char* argv[])
 
   // test references
   std::string str("guest book");
-  sigc::bind (&egon, std::ref(str))(); // Tell bind that it shall store a reference.
+  sigc::bind(&egon, std::ref(str))(); // Tell bind that it shall store a reference.
   result_stream
     << " "
     << str; // (This cannot be the default behaviour: just think about what happens if str dies!)
@@ -146,11 +141,19 @@ main(int argc, char* argv[])
 
   sigc::slot<void()> sl;
   {
+    // Test without a positional template argument to std::bind().
     book guest_book("karl");
     sl = sigc::bind(&egon, std::ref(guest_book));
     sl();
     result_stream << " " << static_cast<std::string&>(guest_book);
     util->check_result(result_stream, "egon(string 'karl') egon was here");
+
+    // Test with a positional template argument to std::bind<>().
+    guest_book.get_name() = "ove";
+    sl = sigc::bind<0>(&egon, std::ref(guest_book));
+    sl();
+    result_stream << " " << static_cast<std::string&>(guest_book);
+    util->check_result(result_stream, "egon(string 'ove') egon was here");
   } // auto-disconnect
 
   sl();
