@@ -5,10 +5,6 @@
 #include "testutilities.h"
 #include <sigc++/adaptors/bind_return.h>
 #include <sigc++/functors/slot.h>
-#include <sstream>
-#include <string>
-#include <functional> //For std::ref().
-#include <cstdlib>
 
 namespace
 {
