@@ -11,9 +11,6 @@ std::ostringstream result_stream;
 
 struct foo
 {
-  // choose a type that can hold all return values
-  using result_type = int;
-
   int operator()()
   {
     result_stream << "foo() ";
@@ -29,8 +26,6 @@ struct foo
 
 struct foo_void
 {
-  using result_type = void;
-
   void operator()() { result_stream << "foo_void()"; }
 };
 
