@@ -11,6 +11,13 @@ std::ostringstream result_stream;
 
 class Base : virtual public sigc::trackable
 {
+public:
+  Base() {}
+
+  Base(const Base& src) = default;
+  Base& operator=(const Base& src) = default;
+  Base(Base&& src) = delete;
+  Base& operator=(Base&& src) = delete;
 };
 
 class Base2
@@ -19,7 +26,9 @@ public:
   virtual ~Base2() {}
 };
 
-class Derived : virtual public Base, public Base2
+class Derived
+: virtual public Base
+, public Base2
 {
 public:
   void method() { result_stream << "method()"; }
@@ -40,11 +49,11 @@ main(int argc, char* argv[])
   handler();
   util->check_result(result_stream, "method()");
 
-  auto param = sigc::bind(sigc::slot<void(Derived&)>(), std::ref(*instance));
+  sigc::slot<void()> param = sigc::bind(sigc::slot<void(Derived&)>(), std::ref(*instance));
   param();
   util->check_result(result_stream, "");
 
-  auto ret = sigc::bind_return(sigc::slot<void()>(), std::ref(*instance));
+  sigc::slot<Derived()> ret = sigc::bind_return(sigc::slot<void()>(), std::ref(*instance));
   ret();
   util->check_result(result_stream, "");
 
